@@ -1,6 +1,6 @@
 /***************************************************************************
     begin       : Tue Dec 31 2013
-    copyright   : (C) 2018 by Martin Preuss
+    copyright   : (C) 2022 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -15,19 +15,12 @@
 
 #include "jobtransferbase_p.h"
 #include "aqhbci/aqhbci_l.h"
-#include "accountjob_l.h"
-#include "aqhbci/joblayer/job_l.h"
 #include "aqhbci/joblayer/job_swift.h"
 #include "aqhbci/joblayer/job_crypt.h"
-#include "aqhbci/banking/provider_l.h"
 #include "aqhbci/applayer/hhd_l.h"
 
-#include <aqbanking/types/transaction.h>
-
 #include <gwenhywfar/debug.h>
-#include <gwenhywfar/misc.h>
 #include <gwenhywfar/inherit.h>
-#include <gwenhywfar/text.h>
 #include <gwenhywfar/gui.h>
 
 #include <assert.h>
@@ -44,10 +37,27 @@ GWEN_INHERIT(AH_JOB, AH_JOB_TRANSFERBASE);
  * ------------------------------------------------------------------------------------------------
  */
 
+static void GWENHYWFAR_CB _freeData(void *bp, void *p);
+static int _process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx);
+static int _handleResults(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx);
 
+/**
+ * Set given status on all transfers and add copies of them to the given context.
+ */
+static void _setStatusOnTransfersAndAddToCtx(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx, AB_TRANSACTION_STATUS status);
+
+static int _exportTransactions(const AH_JOB *j, const AH_JOB_TRANSFERBASE *aj);
+static GWEN_BUFFER *_exportTransactionsToBuffer(const AH_JOB *j, const AH_JOB_TRANSFERBASE *aj, GWEN_DB_NODE *dbProfile);
 static void _replaceCtrlCharsInPurpose(AB_TRANSACTION *t);
 static void _setProfileName(AH_JOB *j, const char *s);
 static void _setDescriptor(AH_JOB *j, const char *s);
+static void _setLimitsCycleMonth(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams);
+static void _setLimitsExecDaysOfMonth(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams);
+static void _setLimitsCycleWeek(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams);
+static void _setLimitsExecDaysOfWeek(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams);
+
+static AB_VALUE *_sumUpTransfers(const AH_JOB *j);
+static int _storeValueInArgs(AH_JOB *j, const AB_VALUE *v, const char *valueGroup);
 
 
 
@@ -58,7 +68,6 @@ static void _setDescriptor(AH_JOB *j, const char *s);
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 AH_JOB *AH_Job_TransferBase_new(const char *jobName,
                                 AB_TRANSACTION_TYPE tt,
                                 AB_TRANSACTION_SUBTYPE tst,
@@ -74,26 +83,26 @@ AH_JOB *AH_Job_TransferBase_new(const char *jobName,
     return 0;
 
   GWEN_NEW_OBJECT(AH_JOB_TRANSFERBASE, aj);
-  GWEN_INHERIT_SETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j, aj, AH_Job_TransferBase_FreeData);
+  GWEN_INHERIT_SETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j, aj, _freeData);
 
   aj->transactionType=tt;
   aj->transactionSubType=tst;
 
   /* overwrite some virtual functions */
-  AH_Job_SetProcessFn(j, AH_Job_TransferBase_Process);
-  AH_Job_SetHandleResultsFn(j, AH_Job_TransferBase_HandleResults);
+  AH_Job_SetProcessFn(j, _process);
+  AH_Job_SetHandleResultsFn(j, _handleResults);
 
   return j;
 }
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-void GWENHYWFAR_CB AH_Job_TransferBase_FreeData(void *bp, void *p)
+void GWENHYWFAR_CB _freeData(void *bp, void *p)
 {
   AH_JOB_TRANSFERBASE *aj;
 
   aj=(AH_JOB_TRANSFERBASE *)p;
+  AB_Value_free(aj->sumValues);
   free(aj->localInstrumentationCode);
   free(aj->profileName);
   free(aj->fiid);
@@ -103,7 +112,6 @@ void GWENHYWFAR_CB AH_Job_TransferBase_FreeData(void *bp, void *p)
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 const char *AH_Job_TransferBase_GetFiid(const AH_JOB *j)
 {
   AH_JOB_TRANSFERBASE *aj;
@@ -117,16 +125,38 @@ const char *AH_Job_TransferBase_GetFiid(const AH_JOB *j)
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
+AB_VALUE *AH_Job_TransferBase_GetSumValues(const AH_JOB *j)
+{
+  if (j) {
+    AH_JOB_TRANSFERBASE *aj;
+
+    aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j);
+    assert(aj);
+    return aj->sumValues;
+  }
+  return NULL;
+}
+
+
+
+void AH_Job_TransferBase_SetSumValues(AH_JOB *j, const AB_VALUE *v)
+{
+  if (j) {
+    AH_JOB_TRANSFERBASE *aj;
+
+    aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j);
+    assert(aj);
+    AB_Value_free(aj->sumValues);
+    aj->sumValues=v?AB_Value_dup(v):NULL;
+  }
+}
+
+
+
 int AH_Job_TransferBase_SepaExportTransactions(AH_JOB *j)
 {
   AH_JOB_TRANSFERBASE *aj;
-  GWEN_DB_NODE *dbArgs;
-  AB_BANKING *ab;
-  const AB_TRANSACTION *t;
   int rv;
-  AB_ACCOUNT *a;
-  GWEN_DB_NODE *dbProfile;
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "Exporting transaction");
 
@@ -134,15 +164,6 @@ int AH_Job_TransferBase_SepaExportTransactions(AH_JOB *j)
   aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j);
   assert(aj);
 
-  ab=AH_Job_GetBankingApi(j);
-  assert(ab);
-
-  dbArgs=AH_Job_GetArguments(j);
-  assert(dbArgs);
-
-  a=AH_AccountJob_GetAccount(j);
-  assert(a);
-
   if (aj->profileName==NULL) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No profile set. SNH!!");
     return GWEN_ERROR_INTERNAL;
@@ -154,30 +175,84 @@ int AH_Job_TransferBase_SepaExportTransactions(AH_JOB *j)
   }
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "Using SEPA descriptor %s and profile %s", aj->descriptor, aj->profileName);
-  GWEN_Gui_ProgressLog2(0,
-                        GWEN_LoggerLevel_Notice,
-                        I18N("Using SEPA descriptor %s and profile %s"),
-                        aj->descriptor, aj->profileName);
-
-  dbProfile=AB_Banking_GetImExporterProfile(ab, "xml", aj->profileName);
-  if (dbProfile==NULL) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Profile \"%s\" not found.", aj->profileName);
-    return GWEN_ERROR_INTERNAL;
-  }
-
-  if (aj->localInstrumentationCode)
-    GWEN_DB_SetCharValue(dbProfile, GWEN_DB_FLAGS_OVERWRITE_VARS, "LocalInstrumentSEPACode", aj->localInstrumentationCode);
+  GWEN_Gui_ProgressLog2(0, GWEN_LoggerLevel_Notice, I18N("Using SEPA descriptor %s and profile %s"), aj->descriptor, aj->profileName);
 
   /* set data in job */
+  rv=_exportTransactions(j, aj);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  return 0;
+}
+
+
+
+int _exportTransactions(const AH_JOB *j, const AH_JOB_TRANSFERBASE *aj)
+{
+  const AB_TRANSACTION *t;
+
   t=AH_Job_GetFirstTransfer(j);
   if (t) {
-    AB_IMEXPORTER_CONTEXT *ioc;
-    AB_TRANSACTION *cpy;
+    GWEN_DB_NODE *dbArgs;
+    AB_BANKING *ab;
+    GWEN_DB_NODE *dbProfile;
     GWEN_BUFFER *dbuf;
 
+    dbArgs=AH_Job_GetArguments(j);
+    ab=AH_Job_GetBankingApi(j);
+
+    dbProfile=AB_Banking_GetImExporterProfile(ab, "xml", aj->profileName);
+    if (dbProfile==NULL) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "Profile \"%s\" not found.", aj->profileName);
+      return GWEN_ERROR_INTERNAL;
+    }
+
+    if (aj->localInstrumentationCode)
+      GWEN_DB_SetCharValue(dbProfile, GWEN_DB_FLAGS_OVERWRITE_VARS, "LocalInstrumentSEPACode", aj->localInstrumentationCode);
+
+    dbuf=_exportTransactionsToBuffer(j, aj, dbProfile);
+    if (dbuf==NULL) {
+      GWEN_DB_Group_free(dbProfile);
+      return GWEN_ERROR_GENERIC;
+    }
+
+    /* store descriptor */
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS, "descriptor", aj->descriptor);
+    /* store transfer */
+    GWEN_DB_SetBinValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS, "transfer", GWEN_Buffer_GetStart(dbuf), GWEN_Buffer_GetUsedBytes(dbuf));
+    GWEN_Buffer_free(dbuf);
+    GWEN_DB_Group_free(dbProfile);
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "No transaction in job");
+    return GWEN_ERROR_INTERNAL;
+  }
+  return 0;
+}
+
+
+
+GWEN_BUFFER *_exportTransactionsToBuffer(const AH_JOB *j, const AH_JOB_TRANSFERBASE *aj, GWEN_DB_NODE *dbProfile)
+{
+  const AB_TRANSACTION *t;
+
+  t=AH_Job_GetFirstTransfer(j);
+  if (t) {
+    AB_BANKING *ab;
+    AB_ACCOUNT *a;
+    AB_IMEXPORTER_CONTEXT *ioc;
+    GWEN_BUFFER *dbuf;
+    int rv;
+
+    ab=AH_Job_GetBankingApi(j);
+    a=AH_AccountJob_GetAccount(j);
     /* add copies of transfers */
     ioc=AB_ImExporterContext_new();
     while (t) {
+      AB_TRANSACTION *cpy;
+
       cpy=AB_Transaction_dup(t);
       _replaceCtrlCharsInPurpose(cpy);
       AB_Transaction_SetUniqueAccountId(cpy, AB_Account_GetUniqueId(a));
@@ -191,30 +266,19 @@ int AH_Job_TransferBase_SepaExportTransactions(AH_JOB *j)
     if (rv<0) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
       GWEN_Buffer_free(dbuf);
-      GWEN_DB_Group_free(dbProfile);
-      return rv;
+      return NULL;
     }
 
-    /* store descriptor */
-    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS, "descriptor", aj->descriptor);
-    /* store transfer */
-    GWEN_DB_SetBinValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS, "transfer", GWEN_Buffer_GetStart(dbuf),
-                        GWEN_Buffer_GetUsedBytes(dbuf));
-    GWEN_Buffer_free(dbuf);
-    GWEN_DB_Group_free(dbProfile);
+    return dbuf;
   }
   else {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No transaction in job");
-    GWEN_DB_Group_free(dbProfile);
-    return GWEN_ERROR_INTERNAL;
+    return NULL;
   }
-
-  return 0;
 }
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 void _replaceCtrlCharsInPurpose(AB_TRANSACTION *trans)
 {
   const char *s;
@@ -239,7 +303,6 @@ void _replaceCtrlCharsInPurpose(AB_TRANSACTION *trans)
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_GetLimits_SepaUndated(AH_JOB *j, AB_TRANSACTION_LIMITS **pLimits)
 {
   AB_TRANSACTION_LIMITS *lim;
@@ -312,12 +375,10 @@ int AH_Job_TransferBase_GetLimits_SepaDated(AH_JOB *j, AB_TRANSACTION_LIMITS **p
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_GetLimits_SepaStandingOrder(AH_JOB *j, AB_TRANSACTION_LIMITS **pLimits)
 {
   AB_TRANSACTION_LIMITS *lim;
   GWEN_DB_NODE *dbParams;
-  const char *s;
   int i;
 
   dbParams=AH_Job_GetParams(j);
@@ -335,120 +396,11 @@ int AH_Job_TransferBase_GetLimits_SepaStandingOrder(AH_JOB *j, AB_TRANSACTION_LI
     AB_TransactionLimits_SetMaxLenRemoteName(lim, 70);
 
     /* get specific limits for creation of standing orders */
-    AB_TransactionLimits_PresetValuesCycleMonth(lim, 0);
-    AB_TransactionLimits_SetValuesCycleMonthUsed(lim, 0);
-    s=GWEN_DB_GetCharValue(dbParams, "AllowedTurnusMonths", 0, 0);
-    if (s && *s) {
-      AB_TransactionLimits_SetAllowMonthly(lim, 1);
-      while (*s) {
-        char buf[3];
-        const char *x;
-        int rv;
-        int d;
+    _setLimitsCycleMonth(lim, dbParams);
+    _setLimitsExecDaysOfMonth(lim, dbParams);
 
-        buf[2]=0;
-        strncpy(buf, s, 2);
-        x=buf;
-        if (*x=='0')
-          x++;
-
-        rv=sscanf(x, "%d", &d);
-        if (rv!=1) {
-          DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params (%s)", x);
-        }
-        else
-          AB_TransactionLimits_ValuesCycleMonthAdd(lim, d);
-        s+=2;
-      } /* while */
-    }
-    else
-      AB_TransactionLimits_SetAllowMonthly(lim, -1);
-
-    AB_TransactionLimits_PresetValuesExecutionDayMonth(lim, 0);
-    AB_TransactionLimits_SetValuesExecutionDayMonthUsed(lim, 0);
-    s=GWEN_DB_GetCharValue(dbParams, "AllowedMonthDays", 0, 0);
-    if (s && *s) {
-      while (*s) {
-        char buf[3];
-        const char *x;
-        int rv;
-        int d;
-
-        buf[2]=0;
-        strncpy(buf, s, 2);
-        x=buf;
-        if (*x=='0')
-          x++;
-
-        rv=sscanf(x, "%d", &d);
-        if (rv!=1) {
-          DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params (%s)", x);
-        }
-        else
-          AB_TransactionLimits_ValuesExecutionDayMonthAdd(lim, d);
-        s+=2;
-      } /* while */
-    }
-
-    AB_TransactionLimits_PresetValuesCycleWeek(lim, 0);
-    AB_TransactionLimits_SetValuesCycleWeekUsed(lim, 0);
-    s=GWEN_DB_GetCharValue(dbParams, "AllowedTurnusWeeks", 0, 0);
-    if (s && *s) {
-      AB_TransactionLimits_SetAllowWeekly(lim, 1);
-      while (*s) {
-        char buf[3];
-        const char *x;
-        int rv;
-        int d;
-
-        buf[2]=0;
-        strncpy(buf, s, 2);
-        x=buf;
-        if (*x=='0')
-          x++;
-
-        rv=sscanf(x, "%d", &d);
-        if (rv!=1) {
-          DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params (%s)", x);
-        }
-        else
-          AB_TransactionLimits_ValuesCycleWeekAdd(lim, d);
-        s+=2;
-      } /* while */
-    }
-    else
-      AB_TransactionLimits_SetAllowWeekly(lim, -1);
-
-    AB_TransactionLimits_PresetValuesExecutionDayWeek(lim, 0);
-    AB_TransactionLimits_SetValuesExecutionDayWeekUsed(lim, 0);
-    s=GWEN_DB_GetCharValue(dbParams, "AllowedWeekDays", 0, 0);
-    if (s && *s) {
-      while (*s) {
-        char buf[2];
-        const char *x;
-        int rv;
-        int d;
-
-        buf[0]=*s;
-        buf[1]=0;
-        x=buf;
-        if (*x=='0')
-          x++;
-
-        if (*x) {
-          rv=sscanf(x, "%d", &d);
-          if (rv!=1) {
-            DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params (current is [%s],  remaining string is [%s])", x, s);
-          }
-          else
-            AB_TransactionLimits_ValuesExecutionDayWeekAdd(lim, d);
-        }
-        else {
-          DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid empty param starting with [%s])", s);
-        }
-        s++;
-      } /* while */
-    }
+    _setLimitsCycleWeek(lim, dbParams);
+    _setLimitsExecDaysOfWeek(lim, dbParams);
   }
 
   i=GWEN_DB_GetIntValue(dbParams, "mindelay", 0, 0);
@@ -467,7 +419,6 @@ int AH_Job_TransferBase_GetLimits_SepaStandingOrder(AH_JOB *j, AB_TRANSACTION_LI
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_HandleCommand_SepaUndated(AH_JOB *j, const AB_TRANSACTION *t)
 {
   AB_TRANSACTION_LIMITS *lim=NULL;
@@ -477,7 +428,7 @@ int AH_Job_TransferBase_HandleCommand_SepaUndated(AH_JOB *j, const AB_TRANSACTIO
   AB_USER *u;
   uint32_t uflags;
 
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Exchanging args");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Handling command");
 
   ab=AH_Job_GetBankingApi(j);
   assert(ab);
@@ -534,7 +485,6 @@ int AH_Job_TransferBase_HandleCommand_SepaUndated(AH_JOB *j, const AB_TRANSACTIO
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_HandleCommand_SepaDated(AH_JOB *j, const AB_TRANSACTION *t)
 {
   AB_TRANSACTION_LIMITS *lim=NULL;
@@ -544,7 +494,7 @@ int AH_Job_TransferBase_HandleCommand_SepaDated(AH_JOB *j, const AB_TRANSACTION
   AB_USER *u;
   uint32_t uflags;
 
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Exchanging args");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Handling command");
 
   ab=AH_Job_GetBankingApi(j);
   assert(ab);
@@ -609,7 +559,6 @@ int AH_Job_TransferBase_HandleCommand_SepaDated(AH_JOB *j, const AB_TRANSACTION
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_HandleCommand_SepaDatedDebit(AH_JOB *j, const AB_TRANSACTION *t)
 {
   AB_TRANSACTION_LIMITS *lim=NULL;
@@ -619,7 +568,7 @@ int AH_Job_TransferBase_HandleCommand_SepaDatedDebit(AH_JOB *j, const AB_TRANSAC
   AB_USER *u;
   uint32_t uflags;
 
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Exchanging args");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Handling command");
 
   ab=AH_Job_GetBankingApi(j);
   assert(ab);
@@ -684,7 +633,6 @@ int AH_Job_TransferBase_HandleCommand_SepaDatedDebit(AH_JOB *j, const AB_TRANSAC
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_HandleCommand_SepaStandingOrder(AH_JOB *j, const AB_TRANSACTION *t)
 {
   AB_TRANSACTION_LIMITS *lim=NULL;
@@ -695,7 +643,7 @@ int AH_Job_TransferBase_HandleCommand_SepaStandingOrder(AH_JOB *j, const AB_TRAN
   uint32_t uflags;
   const char *s;
 
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Exchanging args");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Handling command");
 
   ab=AH_Job_GetBankingApi(j);
   assert(ab);
@@ -772,7 +720,128 @@ int AH_Job_TransferBase_HandleCommand_SepaStandingOrder(AH_JOB *j, const AB_TRAN
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
+int AH_Job_TransferBase_Prepare_SepaStandingOrder(AH_JOB *j)
+{
+  GWEN_DB_NODE *dbArgs;
+  int rv;
+  const GWEN_DATE *da;
+  GWEN_BUFFER *tbuf;
+  const char *s;
+  AB_TRANSACTION *t;
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Preparing transfer");
+
+  dbArgs=AH_Job_GetArguments(j);
+
+  t=AH_Job_GetFirstTransfer(j);
+  if (t==NULL) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "No transaction in job");
+    assert(t); /* debug */
+    return GWEN_ERROR_INTERNAL;
+  }
+
+  /* select pain profile from group "001" */
+  rv=AH_Job_TransferBase_SelectPainProfile(j, 1);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  /* export transfers to SEPA */
+  rv=AH_Job_TransferBase_SepaExportTransactions(j);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  /* execution date */
+  tbuf=GWEN_Buffer_new(0, 16, 0, 1);
+  da=AB_Transaction_GetDate(t);
+  if (da) {
+    GWEN_Date_toStringWithTemplate(da, "YYYYMMDD", tbuf);
+    GWEN_DB_SetCharValue(dbArgs,
+                         GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "xnextExecutionDate",
+                         GWEN_Buffer_GetStart(tbuf));
+    GWEN_Buffer_Reset(tbuf);
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Missing execution date.");
+  }
+
+  /* first execution date */
+  da=AB_Transaction_GetFirstDate(t);
+  if (da) {
+    GWEN_Date_toStringWithTemplate(da, "YYYYMMDD", tbuf);
+    GWEN_DB_SetCharValue(dbArgs,
+                         GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "details/xfirstExecutionDate",
+                         GWEN_Buffer_GetStart(tbuf));
+    GWEN_Buffer_Reset(tbuf);
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Missing first execution date.");
+  }
+
+  /* last execution date */
+  da=AB_Transaction_GetLastDate(t);
+  if (da) {
+    GWEN_Date_toStringWithTemplate(da, "YYYYMMDD", tbuf);
+    GWEN_DB_SetCharValue(dbArgs,
+                         GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "details/xlastExecutionDate",
+                         GWEN_Buffer_GetStart(tbuf));
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Missing last execution date.");
+  }
+  GWEN_Buffer_free(tbuf);
+
+  /* period */
+  switch (AB_Transaction_GetPeriod(t)) {
+  case AB_Transaction_PeriodMonthly:
+    s="M";
+    break;
+  case AB_Transaction_PeriodWeekly:
+    s="W";
+    break;
+  default:
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Unsupported period %d",
+              AB_Transaction_GetPeriod(t));
+    return GWEN_ERROR_INVALID;
+  }
+  GWEN_DB_SetCharValue(dbArgs,
+                       GWEN_DB_FLAGS_OVERWRITE_VARS,
+                       "details/xperiod",
+                       s);
+
+  /* cycle */
+  GWEN_DB_SetIntValue(dbArgs,
+                      GWEN_DB_FLAGS_OVERWRITE_VARS,
+                      "details/cycle",
+                      AB_Transaction_GetCycle(t));
+
+  /* execution day */
+  GWEN_DB_SetIntValue(dbArgs,
+                      GWEN_DB_FLAGS_OVERWRITE_VARS,
+                      "details/executionDay",
+                      AB_Transaction_GetExecutionDay(t));
+
+
+  /* SET fiId, if present */
+  s=AB_Transaction_GetFiId(t);
+  if (s) {
+    GWEN_DB_SetCharValue(dbArgs,
+                         GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "fiId",
+                         AB_Transaction_GetFiId(t));
+  }
+
+  return 0;
+}
+
+
+
 int AH_Job_TransferBase_AddChallengeParams29(AH_JOB *j, int hkTanVer, GWEN_DB_NODE *dbMethod)
 {
   const AB_TRANSACTION *t;
@@ -789,14 +858,14 @@ int AH_Job_TransferBase_AddChallengeParams29(AH_JOB *j, int hkTanVer, GWEN_DB_NO
 
   s=GWEN_DB_GetCharValue(dbMethod, "zkaTanVersion", 0, NULL);
   if (s && *s && strncasecmp(s, "1.3", 3)==0) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.3 (%s)", s);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "TAN version is 1.3 (%s)", s);
     tanVer=AH_JOB_TANVER_1_3;
   }
 
   if (tanVer==AH_JOB_TANVER_1_4) {
     int rv;
 
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.4.x");
+    DBG_INFO(AQHBCI_LOGDOMAIN, "TAN version is 1.4.x");
     rv=AH_HHD14_AddChallengeParams_29(j,
                                       AB_Transaction_GetValue(t),
                                       AB_Transaction_GetRemoteIban(t),
@@ -815,15 +884,12 @@ int AH_Job_TransferBase_AddChallengeParams29(AH_JOB *j, int hkTanVer, GWEN_DB_NO
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
 int AH_Job_TransferBase_AddChallengeParams35(AH_JOB *j, int hkTanVer, GWEN_DB_NODE *dbMethod)
 {
   const AB_TRANSACTION *t;
   const char *s;
   int tanVer=AH_JOB_TANVER_1_4;
 
-  DBG_ERROR(AQHBCI_LOGDOMAIN, "AddChallengeParams function called");
-
   t=AH_Job_GetFirstTransfer(j);
   if (t==NULL) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No validated transaction");
@@ -832,14 +898,14 @@ int AH_Job_TransferBase_AddChallengeParams35(AH_JOB *j, int hkTanVer, GWEN_DB_NO
 
   s=GWEN_DB_GetCharValue(dbMethod, "zkaTanVersion", 0, NULL);
   if (s && *s && strncasecmp(s, "1.3", 3)==0) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.3 (%s)", s);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "TAN version is 1.3 (%s)", s);
     tanVer=AH_JOB_TANVER_1_3;
   }
 
   if (tanVer==AH_JOB_TANVER_1_4) {
     int rv;
 
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.4.x");
+    DBG_INFO(AQHBCI_LOGDOMAIN, "TAN version is 1.4.x");
     rv=AH_HHD14_AddChallengeParams_35(j,
                                       AB_Transaction_GetValue(t),
                                       AB_Transaction_GetRemoteIban(t));
@@ -857,9 +923,7 @@ int AH_Job_TransferBase_AddChallengeParams35(AH_JOB *j, int hkTanVer, GWEN_DB_NO
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-void AH_Job_TransferBase_SetStatusOnTransfersAndAddToCtx(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx,
-                                                         AB_TRANSACTION_STATUS status)
+void _setStatusOnTransfersAndAddToCtx(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx, AB_TRANSACTION_STATUS status)
 {
   AH_JOB_TRANSFERBASE *aj;
   const AB_TRANSACTION *t;
@@ -899,8 +963,7 @@ void AH_Job_TransferBase_SetStatusOnTransfersAndAddToCtx(AH_JOB *j, AB_IMEXPORTE
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-int AH_Job_TransferBase_HandleResults(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
+int _handleResults(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
 {
   AH_JOB_TRANSFERBASE *aj;
   AH_RESULT_LIST *rl;
@@ -946,15 +1009,14 @@ int AH_Job_TransferBase_HandleResults(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
       tStatus=AB_Transaction_StatusRejected;
   }
 
-  AH_Job_TransferBase_SetStatusOnTransfersAndAddToCtx(j, ctx, tStatus);
+  _setStatusOnTransfersAndAddToCtx(j, ctx, tStatus);
   AH_Job_SetStatusOnCommands(j, tStatus);
   return 0;
 }
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-int AH_Job_TransferBase_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
+int _process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
 {
   AH_JOB_TRANSFERBASE *aj;
   GWEN_DB_NODE *dbResponses;
@@ -1033,7 +1095,7 @@ int AH_Job_TransferBase_SelectPainProfile(AH_JOB *j, int version1)
     assert(descr);
 
     s=AB_SwiftDescr_GetAlias2(descr);
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Selecting PAIN format [%s]", s);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Selecting PAIN format [%s]", s);
     _setDescriptor(j, s);
     _setProfileName(j, AB_SwiftDescr_GetAlias1(descr));
 
@@ -1103,5 +1165,227 @@ void _setDescriptor(AH_JOB *j, const char *s)
 
 
 
+void _setLimitsCycleMonth(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams)
+{
+  const char *s;
+
+  AB_TransactionLimits_PresetValuesCycleMonth(lim, 0);
+  AB_TransactionLimits_SetValuesCycleMonthUsed(lim, 0);
+  s=GWEN_DB_GetCharValue(dbParams, "AllowedTurnusMonths", 0, 0);
+  if (s && *s) {
+    AB_TransactionLimits_SetAllowMonthly(lim, 1);
+
+    while (*s) {
+      /* read 2-digit number */
+      if (isdigit(*s) && isdigit(s[1]))
+        AB_TransactionLimits_ValuesCycleMonthAdd(lim, ((int)((s[0]-'0')*10))+((int)(s[1]-'0')));
+      else {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params [%s]", s);
+      }
+      s+=2;
+    } /* while */
+  }
+  else
+    AB_TransactionLimits_SetAllowMonthly(lim, -1);
+}
+
+
+
+void _setLimitsExecDaysOfMonth(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams)
+{
+  const char *s;
+
+  AB_TransactionLimits_PresetValuesExecutionDayMonth(lim, 0);
+  AB_TransactionLimits_SetValuesExecutionDayMonthUsed(lim, 0);
+  s=GWEN_DB_GetCharValue(dbParams, "AllowedMonthDays", 0, 0);
+  if (s && *s) {
+    while (*s) {
+      /* read 2-digit number */
+      if (isdigit(*s) && isdigit(s[1]))
+        AB_TransactionLimits_ValuesExecutionDayMonthAdd(lim, ((int)((s[0]-'0')*10))+((int)(s[1]-'0')));
+      else {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params [%s]", s);
+      }
+      s+=2;
+    } /* while */
+  }
+}
+
+
+
+void _setLimitsCycleWeek(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams)
+{
+  const char *s;
+
+  AB_TransactionLimits_PresetValuesCycleWeek(lim, 0);
+  AB_TransactionLimits_SetValuesCycleWeekUsed(lim, 0);
+  s=GWEN_DB_GetCharValue(dbParams, "AllowedTurnusWeeks", 0, 0);
+  if (s && *s) {
+    AB_TransactionLimits_SetAllowWeekly(lim, 1);
+    while (*s) {
+      /* read 2-digit number */
+      if (isdigit(*s) && isdigit(s[1]))
+        AB_TransactionLimits_ValuesCycleWeekAdd(lim, ((int)((s[0]-'0')*10))+((int)(s[1]-'0')));
+      else {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params [%s]", s);
+      }
+      s+=2;
+    } /* while */
+  }
+  else
+    AB_TransactionLimits_SetAllowWeekly(lim, -1);
+}
+
+
+
+void _setLimitsExecDaysOfWeek(AB_TRANSACTION_LIMITS *lim, GWEN_DB_NODE *dbParams)
+{
+  const char *s;
+
+  AB_TransactionLimits_PresetValuesExecutionDayWeek(lim, 0);
+  AB_TransactionLimits_SetValuesExecutionDayWeekUsed(lim, 0);
+  s=GWEN_DB_GetCharValue(dbParams, "AllowedWeekDays", 0, 0);
+  if (s && *s) {
+    while (*s) {
+      if (isdigit(*s))
+        AB_TransactionLimits_ValuesExecutionDayWeekAdd(lim, (int)((*s)-'0')); /* 1-digit number */
+      else {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Invalid number in params [%s]", s);
+      }
+      s++;
+    }
+  }
+  else {
+    AB_TransactionLimits_SetAllowWeekly(lim, -1);
+  }
+}
+
+
+
+int AH_Job_TransferBase_Prepare(AH_JOB *j, int painProfile, const char *localInstrumentationCode, int singleBookingAllowed)
+{
+  AH_JOB_TRANSFERBASE *aj;
+
+  assert(j);
+  aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_TRANSFERBASE, j);
+  assert(aj);
+
+  if (AH_Job_GetMsgNum(j)==0) { /* only when called for the first time */
+    GWEN_DB_NODE *dbArgs;
+    int rv;
+
+    dbArgs=AH_Job_GetArguments(j);
+
+    /* set singleBookingWanted */
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS, "singleBookingWanted", (singleBookingAllowed)?"J":"N");
+
+    /* calculate sum */
+    AB_Value_free(aj->sumValues);
+    aj->sumValues=_sumUpTransfers(j);
+    if (aj->sumValues==NULL) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "Error summing up transfers.");
+      return GWEN_ERROR_INTERNAL;
+    }
+
+    rv=AH_Job_TransferBase_SelectPainProfile(j, painProfile);
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+      return rv;
+    }
+
+    if (localInstrumentationCode && *localInstrumentationCode)
+      AH_Job_TransferBase_SetLocalInstrumentationCode(j, localInstrumentationCode);
+
+    /* export transfers to SEPA */
+    rv=AH_Job_TransferBase_SepaExportTransactions(j);
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+      return rv;
+    }
+
+    /* store sum value */
+    rv=_storeValueInArgs(j, aj->sumValues, "totalSum");
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+      return rv;
+    }
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Job already prepared");
+  }
+  return 0;
+}
+
+
+
+
+AB_VALUE *_sumUpTransfers(const AH_JOB *j)
+{
+  AB_TRANSACTION *t;
+  AB_VALUE *sum;
+
+  assert(j);
+  
+  sum=AB_Value_new();
+  AB_Value_SetCurrency(sum, "EUR");
+  t=AH_Job_GetFirstTransfer(j);
+  if (t==NULL) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "No transaction in job");
+    AB_Value_free(sum);
+    return NULL;
+  }
+  while (t) {
+    const AB_VALUE *v;
+  
+    v=AB_Transaction_GetValue(t);
+    if (v) {
+      const char *s;
+  
+      s=AB_Value_GetCurrency(v);
+      if (s && strcmp(s, "EUR")) {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "EUR required in SEPA transactions (%s)", s);
+        AB_Value_free(sum);
+        return NULL;
+      }
+      AB_Value_AddValue(sum, v);
+    }
+    t=AB_Transaction_List_Next(t);
+  }
+
+  return sum;
+}
+
+
+
+int _storeValueInArgs(AH_JOB *j, const AB_VALUE *v, const char *valueGroup)
+{
+  GWEN_DB_NODE *dbArgs;
+  GWEN_DB_NODE *dbV;
+  GWEN_BUFFER *nbuf;
+  const char *s;
+
+  dbArgs=AH_Job_GetArguments(j);
+  dbV=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_OVERWRITE_GROUPS, valueGroup);
+  assert(dbV);
+
+  nbuf=GWEN_Buffer_new(0, 32, 0, 1);
+  AB_Value_toHbciString(v, nbuf);
+  if (GWEN_Buffer_GetUsedBytes(nbuf)<1) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Error in conversion");
+    GWEN_Buffer_free(nbuf);
+    return GWEN_ERROR_BAD_DATA;
+  }
+
+  /* store value */
+  GWEN_DB_SetCharValue(dbV, GWEN_DB_FLAGS_OVERWRITE_VARS, "value", GWEN_Buffer_GetStart(nbuf));
+  GWEN_Buffer_free(nbuf);
+
+  /* store currency */
+  s=AB_Value_GetCurrency(v);
+  assert(s);
+  GWEN_DB_SetCharValue(dbV, GWEN_DB_FLAGS_OVERWRITE_VARS, "currency", s);
+
+  return 0;
+}
 
 
