@@ -48,17 +48,59 @@
 #define DIALOG_MINHEIGHT 200
 
 
-
-static int _reallyDoIt(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid);
-static int _selectTanMethod(GWEN_DIALOG *dlg, AB_USER *u, int doLock);
-
-
-
-
 GWEN_INHERIT(GWEN_DIALOG, AH_PINTAN_DIALOG)
 
 
 
+/* ------------------------------------------------------------------------------------------------
+ * forward declarations
+ * ------------------------------------------------------------------------------------------------
+ */
+
+static void GWENHYWFAR_CB _dlgApi_FreeData(void *bp, void *p);
+static void _dialogInit(GWEN_DIALOG *dlg);
+static void _dialogFini(GWEN_DIALOG *dlg);
+static int _dialogNext(GWEN_DIALOG *dlg);
+static int _dialogPrevious(GWEN_DIALOG *dlg);
+static int _dialogEnterPage(GWEN_DIALOG *dlg, int page, int forwards);
+
+static int GWENHYWFAR_CB _dlgApi_SignalHandler(GWEN_DIALOG *dlg, GWEN_DIALOG_EVENTTYPE t, const char *sender);
+static int _handleSignalActivated(GWEN_DIALOG *dlg, const char *sender);
+static int _handleSignalActivatedBankCode(GWEN_DIALOG *dlg);
+static int _handleSignalActivatedSpecial(GWEN_DIALOG *dlg);
+static int _handleSignalValueChanged(GWEN_DIALOG *dlg, const char *sender);
+
+static int _getBankPageData(GWEN_DIALOG *dlg);
+static int _getUserPageData(GWEN_DIALOG *dlg);
+
+static void _setBankCode(GWEN_DIALOG *dlg, const char *s);
+static void _setBankName(GWEN_DIALOG *dlg, const char *s);
+static void _setUserName(GWEN_DIALOG *dlg, const char *s);
+static void _setUserId(GWEN_DIALOG *dlg, const char *s);
+static void _setCustomerId(GWEN_DIALOG *dlg, const char *s);
+static void _setUrl(GWEN_DIALOG *dlg, const char *s);
+static void _setTanMediumId(GWEN_DIALOG *dlg, const char *s);
+
+static int _addUserAndSetupWithBankServer(GWEN_DIALOG *dlg);
+static int _setupLockedUserWithBankServer(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid);
+static int _selectTanMethod(GWEN_DIALOG *dlg, AB_USER *u, int doLock);
+static AB_USER *_createAndSetupUser(GWEN_DIALOG *dlg, uint32_t pid);
+
+static int _getCertificate(AB_PROVIDER *pro, AB_USER *u, uint32_t pid);
+static int _getBankInfoAnon(AB_PROVIDER *pro, AB_USER *u, uint32_t pid);
+static int _getSystemId(AB_PROVIDER *pro, AB_USER *u, uint32_t pid);
+static int _getAccountList(AB_PROVIDER *pro, AB_USER *u, uint32_t pid);
+
+static void _removeAllSpaces(uint8_t *s);
+
+
+
+/* ------------------------------------------------------------------------------------------------
+ * implementations
+ * ------------------------------------------------------------------------------------------------
+ */
+
+
 
 GWEN_DIALOG *AH_PinTanDialog_new(AB_PROVIDER *pro)
 {
@@ -74,9 +116,8 @@ GWEN_DIALOG *AH_PinTanDialog_new(AB_PROVIDER *pro)
     return NULL;
   }
   GWEN_NEW_OBJECT(AH_PINTAN_DIALOG, xdlg);
-  GWEN_INHERIT_SETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg, xdlg,
-                       AH_PinTanDialog_FreeData);
-  GWEN_Dialog_SetSignalHandler(dlg, AH_PinTanDialog_SignalHandler);
+  GWEN_INHERIT_SETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg, xdlg, _dlgApi_FreeData);
+  GWEN_Dialog_SetSignalHandler(dlg, _dlgApi_SignalHandler);
 
   xdlg->banking=AB_Provider_GetBanking(pro);
   xdlg->provider=pro;
@@ -92,7 +133,7 @@ GWEN_DIALOG *AH_PinTanDialog_new(AB_PROVIDER *pro)
 
 
 
-void GWENHYWFAR_CB AH_PinTanDialog_FreeData(void *bp, void *p)
+void GWENHYWFAR_CB _dlgApi_FreeData(void *bp, void *p)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -121,20 +162,8 @@ AB_USER *AH_PinTanDialog_GetUser(const GWEN_DIALOG *dlg)
 }
 
 
-const char *AH_PinTanDialog_GetBankCode(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
 
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->bankCode;
-}
-
-
-
-void AH_PinTanDialog_SetBankCode(GWEN_DIALOG *dlg, const char *s)
+void _setBankCode(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -143,28 +172,12 @@ void AH_PinTanDialog_SetBankCode(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->bankCode);
-  if (s)
-    xdlg->bankCode=strdup(s);
-  else
-    xdlg->bankCode=NULL;
+  xdlg->bankCode=s?strdup(s):NULL;
 }
 
 
 
-const char *AH_PinTanDialog_GetBankName(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->bankName;
-}
-
-
-
-void AH_PinTanDialog_SetBankName(GWEN_DIALOG *dlg, const char *s)
+void _setBankName(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -173,28 +186,12 @@ void AH_PinTanDialog_SetBankName(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->bankName);
-  if (s)
-    xdlg->bankName=strdup(s);
-  else
-    xdlg->bankName=NULL;
+  xdlg->bankName=s?strdup(s):NULL;
 }
 
 
 
-const char *AH_PinTanDialog_GetUserName(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->userName;
-}
-
-
-
-void AH_PinTanDialog_SetUserName(GWEN_DIALOG *dlg, const char *s)
+void _setUserName(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -203,28 +200,12 @@ void AH_PinTanDialog_SetUserName(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->userName);
-  if (s)
-    xdlg->userName=strdup(s);
-  else
-    xdlg->userName=NULL;
+  xdlg->userName=s?strdup(s):NULL;
 }
 
 
 
-const char *AH_PinTanDialog_GetUserId(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->userId;
-}
-
-
-
-void AH_PinTanDialog_SetUserId(GWEN_DIALOG *dlg, const char *s)
+void _setUserId(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -233,28 +214,12 @@ void AH_PinTanDialog_SetUserId(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->userId);
-  if (s)
-    xdlg->userId=strdup(s);
-  else
-    xdlg->userId=NULL;
+  xdlg->userId=s?strdup(s):NULL;
 }
 
 
 
-const char *AH_PinTanDialog_GetCustomerId(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->customerId;
-}
-
-
-
-void AH_PinTanDialog_SetCustomerId(GWEN_DIALOG *dlg, const char *s)
+void _setCustomerId(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -263,28 +228,12 @@ void AH_PinTanDialog_SetCustomerId(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->customerId);
-  if (s)
-    xdlg->customerId=strdup(s);
-  else
-    xdlg->customerId=NULL;
+  xdlg->customerId=s?strdup(s):NULL;
 }
 
 
 
-const char *AH_PinTanDialog_GetUrl(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->url;
-}
-
-
-
-void AH_PinTanDialog_SetUrl(GWEN_DIALOG *dlg, const char *s)
+void _setUrl(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -293,146 +242,12 @@ void AH_PinTanDialog_SetUrl(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->url);
-  if (s)
-    xdlg->url=strdup(s);
-  else
-    xdlg->url=NULL;
+  xdlg->url=s?strdup(s):NULL;
 }
 
 
 
-int AH_PinTanDialog_GetHttpVMajor(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->httpVMajor;
-}
-
-
-
-int AH_PinTanDialog_GetHttpVMinor(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->httpVMinor;
-}
-
-
-
-void AH_PinTanDialog_SetHttpVersion(GWEN_DIALOG *dlg, int vmajor, int vminor)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  xdlg->httpVMajor=vmajor;
-  xdlg->httpVMinor=vminor;
-}
-
-
-
-int AH_PinTanDialog_GetHbciVersion(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->hbciVersion;
-}
-
-
-
-void AH_PinTanDialog_SetHbciVersion(GWEN_DIALOG *dlg, int i)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  xdlg->hbciVersion=i;
-}
-
-
-
-uint32_t AH_PinTanDialog_GetFlags(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->flags;
-}
-
-
-
-void AH_PinTanDialog_SetFlags(GWEN_DIALOG *dlg, uint32_t fl)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  xdlg->flags=fl;
-}
-
-
-
-void AH_PinTanDialog_AddFlags(GWEN_DIALOG *dlg, uint32_t fl)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  xdlg->flags&=~fl;
-}
-
-
-
-void AH_PinTanDialog_SubFlags(GWEN_DIALOG *dlg, uint32_t fl)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  xdlg->flags&=~fl;
-}
-
-
-
-const char *AH_PinTanDialog_GetTanMediumId(const GWEN_DIALOG *dlg)
-{
-  AH_PINTAN_DIALOG *xdlg;
-
-  assert(dlg);
-  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
-  assert(xdlg);
-
-  return xdlg->tanMediumId;
-}
-
-
-
-void AH_PinTanDialog_SetTanMediumId(GWEN_DIALOG *dlg, const char *s)
+void _setTanMediumId(GWEN_DIALOG *dlg, const char *s)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -441,21 +256,12 @@ void AH_PinTanDialog_SetTanMediumId(GWEN_DIALOG *dlg, const char *s)
   assert(xdlg);
 
   free(xdlg->tanMediumId);
-  if (s)
-    xdlg->tanMediumId=strdup(s);
-  else
-    xdlg->tanMediumId=NULL;
+  xdlg->tanMediumId=s?strdup(s):NULL;
 }
 
 
 
-
-
-
-
-
-
-void AH_PinTanDialog_Init(GWEN_DIALOG *dlg)
+void _dialogInit(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   GWEN_DB_NODE *dbPrefs;
@@ -559,7 +365,7 @@ void AH_PinTanDialog_Init(GWEN_DIALOG *dlg)
 
 
 
-void AH_PinTanDialog_Fini(GWEN_DIALOG *dlg)
+void _dialogFini(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   int i;
@@ -588,22 +394,7 @@ void AH_PinTanDialog_Fini(GWEN_DIALOG *dlg)
 
 
 
-static void removeAllSpaces(uint8_t *s)
-{
-  uint8_t *d;
-
-  d=s;
-  while (*s) {
-    if (*s>33)
-      *(d++)=*s;
-    s++;
-  }
-  *d=0;
-}
-
-
-
-int AH_PinTanDialog_GetBankPageData(GWEN_DIALOG *dlg)
+int _getBankPageData(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   const char *s;
@@ -619,7 +410,7 @@ int AH_PinTanDialog_GetBankPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    AH_PinTanDialog_SetBankCode(dlg, GWEN_Buffer_GetStart(tbuf));
+    _setBankCode(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else {
@@ -634,11 +425,11 @@ int AH_PinTanDialog_GetBankPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    AH_PinTanDialog_SetBankName(dlg, GWEN_Buffer_GetStart(tbuf));
+    _setBankName(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else
-    AH_PinTanDialog_SetBankName(dlg, NULL);
+    _setBankName(dlg, NULL);
 
   s=GWEN_Dialog_GetCharProperty(dlg, "wiz_url_edit", GWEN_DialogProperty_Value, 0, NULL);
   if (s && *s) {
@@ -647,8 +438,8 @@ int AH_PinTanDialog_GetBankPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    removeAllSpaces((uint8_t *)GWEN_Buffer_GetStart(tbuf));
-    AH_PinTanDialog_SetUrl(dlg, GWEN_Buffer_GetStart(tbuf));
+    _removeAllSpaces((uint8_t *)GWEN_Buffer_GetStart(tbuf));
+    _setUrl(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else {
@@ -661,7 +452,7 @@ int AH_PinTanDialog_GetBankPageData(GWEN_DIALOG *dlg)
 
 
 
-int AH_PinTanDialog_GetUserPageData(GWEN_DIALOG *dlg)
+int _getUserPageData(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   const char *s;
@@ -677,7 +468,7 @@ int AH_PinTanDialog_GetUserPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    AH_PinTanDialog_SetUserName(dlg, GWEN_Buffer_GetStart(tbuf));
+    _setUserName(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else {
@@ -692,7 +483,7 @@ int AH_PinTanDialog_GetUserPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    AH_PinTanDialog_SetUserId(dlg, GWEN_Buffer_GetStart(tbuf));
+    _setUserId(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else {
@@ -707,18 +498,18 @@ int AH_PinTanDialog_GetUserPageData(GWEN_DIALOG *dlg)
     tbuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(tbuf, s);
     GWEN_Text_CondenseBuffer(tbuf);
-    AH_PinTanDialog_SetCustomerId(dlg, GWEN_Buffer_GetStart(tbuf));
+    _setCustomerId(dlg, GWEN_Buffer_GetStart(tbuf));
     GWEN_Buffer_free(tbuf);
   }
   else
-    AH_PinTanDialog_SetCustomerId(dlg, NULL);
+    _setCustomerId(dlg, NULL);
 
   return 0;
 }
 
 
 
-int AH_PinTanDialog_EnterPage(GWEN_DIALOG *dlg, int page, int forwards)
+int _dialogEnterPage(GWEN_DIALOG *dlg, int page, int forwards)
 {
   AH_PINTAN_DIALOG *xdlg;
   int rv;
@@ -735,7 +526,7 @@ int AH_PinTanDialog_EnterPage(GWEN_DIALOG *dlg, int page, int forwards)
 
   case PAGE_BANK:
     GWEN_Dialog_SetIntProperty(dlg, "wiz_prev_button", GWEN_DialogProperty_Enabled, 0, 1, 0);
-    rv=AH_PinTanDialog_GetBankPageData(dlg);
+    rv=_getBankPageData(dlg);
     if (rv<0)
       GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 0, 0);
     else
@@ -745,7 +536,7 @@ int AH_PinTanDialog_EnterPage(GWEN_DIALOG *dlg, int page, int forwards)
 
   case PAGE_USER:
     GWEN_Dialog_SetIntProperty(dlg, "wiz_prev_button", GWEN_DialogProperty_Enabled, 0, 1, 0);
-    rv=AH_PinTanDialog_GetUserPageData(dlg);
+    rv=_getUserPageData(dlg);
     if (rv<0)
       GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 0, 0);
     else
@@ -776,27 +567,113 @@ int AH_PinTanDialog_EnterPage(GWEN_DIALOG *dlg, int page, int forwards)
 
 
 
-int AH_PinTanDialog_DoIt(GWEN_DIALOG *dlg)
+int _addUserAndSetupWithBankServer(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   AB_USER *u;
-  GWEN_URL *url;
   int rv;
   uint32_t pid;
 
-  DBG_NOTICE(0, "Doit");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Create and Setup PinTan HBCI User");
   assert(dlg);
   xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
   assert(xdlg);
 
-  DBG_NOTICE(0, "Creating user");
-  u=AB_Provider_CreateUserObject(xdlg->provider);
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Creating user");
+  u=_createAndSetupUser(dlg, 0);
   if (u==NULL) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not create user, maybe backend missing?");
     // TODO: show error message
     return GWEN_DialogEvent_ResultHandled;
   }
 
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Adding user");
+  rv=AB_Provider_AddUser(xdlg->provider, u);
+  if (rv<0) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not add user (%d)", rv);
+    AB_User_free(u);
+    return GWEN_DialogEvent_ResultHandled;
+  }
+
+  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+                             GWEN_GUI_PROGRESS_ALLOW_EMBED |
+                             GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+                             GWEN_GUI_PROGRESS_SHOW_ABORT,
+                             I18N("Setting Up PIN/TAN User"),
+                             I18N("The system id and a list of accounts will be retrieved."),
+                             4,
+                             0);
+  /* lock new user */
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Locking user");
+  rv=AB_Provider_BeginExclUseUser(xdlg->provider, u);
+  if (rv<0) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not lock user (%d)", rv);
+    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Unable to lock users"));
+    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
+    GWEN_Gui_ProgressEnd(pid);
+    return GWEN_DialogEvent_ResultHandled;
+  }
+
+  /* do all the magic stuff */
+  rv=_setupLockedUserWithBankServer(dlg, u, pid);
+  if (rv<0) {
+    AB_Provider_EndExclUseUser(xdlg->provider, u, 1);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
+    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user."));
+    GWEN_Gui_ProgressEnd(pid);
+    return GWEN_DialogEvent_ResultHandled;
+  }
+
+  /* unlock user */
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Unlocking user");
+  rv=AB_Provider_EndExclUseUser(xdlg->provider, u, 0);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Could not unlock customer [%s] (%d)", AB_User_GetCustomerId(u), rv);
+    GWEN_Gui_ProgressLog2(pid,
+                          GWEN_LoggerLevel_Error,
+                          I18N("Could not unlock user %s (%d)"),
+                          AB_User_GetUserId(u), rv);
+    AB_Provider_EndExclUseUser(xdlg->provider, u, 1);
+    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
+    GWEN_Gui_ProgressEnd(pid);
+    return GWEN_DialogEvent_ResultHandled;
+  }
+
+  GWEN_Dialog_SetCharProperty(dlg,
+                              "wiz_end_label",
+                              GWEN_DialogProperty_Title,
+                              0,
+                              I18N("The user has been successfully setup."),
+                              0);
+  GWEN_Gui_ProgressEnd(pid);
+  _dialogEnterPage(dlg, PAGE_END, 1);
+
+  xdlg->user=u;
+
+  return GWEN_DialogEvent_ResultHandled;
+}
+
+
+
+AB_USER *_createAndSetupUser(GWEN_DIALOG *dlg, uint32_t pid)
+{
+  AH_PINTAN_DIALOG *xdlg;
+  AB_USER *u;
+  GWEN_URL *url;
+
+  assert(dlg);
+  xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
+  assert(xdlg);
+
+  DBG_INFO(0, "Creating user");
+  u=AB_Provider_CreateUserObject(xdlg->provider);
+  if (u==NULL) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not create user, maybe backend missing?");
+    // TODO: show error message
+    return NULL;
+  }
+
   /* generic setup */
   AB_User_SetUserName(u, xdlg->userName);
   AB_User_SetUserId(u, xdlg->userId);
@@ -826,146 +703,37 @@ int AH_PinTanDialog_DoIt(GWEN_DIALOG *dlg)
   AH_User_SetFlags(u, xdlg->flags);
   AH_User_SetTanMediumId(u, xdlg->tanMediumId);
 
-  DBG_NOTICE(0, "Adding user");
-  rv=AB_Provider_AddUser(xdlg->provider, u);
-  if (rv<0) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not add user (%d)", rv);
-    AB_User_free(u);
-    return GWEN_DialogEvent_ResultHandled;
-  }
-
-  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
-                             GWEN_GUI_PROGRESS_ALLOW_EMBED |
-                             GWEN_GUI_PROGRESS_SHOW_PROGRESS |
-                             GWEN_GUI_PROGRESS_SHOW_ABORT,
-                             I18N("Setting Up PIN/TAN User"),
-                             I18N("The system id and a list of accounts will be retrieved."),
-                             4,
-                             0);
-  /* lock new user */
-  DBG_NOTICE(0, "Locking user");
-  rv=AB_Provider_BeginExclUseUser(xdlg->provider, u);
-  if (rv<0) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not lock user (%d)", rv);
-    GWEN_Gui_ProgressLog(pid,
-                         GWEN_LoggerLevel_Error,
-                         I18N("Unable to lock users"));
-    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
-    GWEN_Gui_ProgressEnd(pid);
-    return GWEN_DialogEvent_ResultHandled;
-  }
-
-  /* do all the magic stuff */
-  rv=_reallyDoIt(dlg, u, pid);
-  if (rv<0) {
-    AB_Provider_EndExclUseUser(xdlg->provider, u, 1);
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
-    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error, I18N("Aborted by user."));
-    GWEN_Gui_ProgressEnd(pid);
-    return GWEN_DialogEvent_ResultHandled;
-  }
-
-  /* unlock user */
-  DBG_NOTICE(0, "Unlocking user");
-  rv=AB_Provider_EndExclUseUser(xdlg->provider, u, 0);
-  if (rv<0) {
-    DBG_INFO(AQHBCI_LOGDOMAIN,
-             "Could not unlock customer [%s] (%d)",
-             AB_User_GetCustomerId(u), rv);
-    GWEN_Gui_ProgressLog2(pid,
-                          GWEN_LoggerLevel_Error,
-                          I18N("Could not unlock user %s (%d)"),
-                          AB_User_GetUserId(u), rv);
-    AB_Provider_EndExclUseUser(xdlg->provider, u, 1);
-    AB_Provider_DeleteUser(xdlg->provider, AB_User_GetUniqueId(u));
-    GWEN_Gui_ProgressEnd(pid);
-    return GWEN_DialogEvent_ResultHandled;
-  }
-
-  GWEN_Dialog_SetCharProperty(dlg,
-                              "wiz_end_label",
-                              GWEN_DialogProperty_Title,
-                              0,
-                              I18N("The user has been successfully setup."),
-                              0);
-  GWEN_Gui_ProgressEnd(pid);
-  AH_PinTanDialog_EnterPage(dlg, PAGE_END, 1);
-
-  xdlg->user=u;
-
-  return GWEN_DialogEvent_ResultHandled;
+  return u;
 }
 
 
 
-
-int _reallyDoIt(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid)
+int _setupLockedUserWithBankServer(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid)
 {
   AH_PINTAN_DIALOG *xdlg;
   int rv;
-  AB_IMEXPORTER_CONTEXT *ctx;
 
   assert(dlg);
   xdlg=GWEN_INHERIT_GETDATA(GWEN_DIALOG, AH_PINTAN_DIALOG, dlg);
   assert(xdlg);
 
-
-  /* get certificate */
-  DBG_NOTICE(0, "Getting certs (%08x)", AH_User_GetFlags(u));
-  GWEN_Gui_ProgressLog(pid,
-                       GWEN_LoggerLevel_Notice,
-                       I18N("Retrieving SSL certificate"));
-  rv=AH_Provider_GetCert(xdlg->provider, u, 0, 1, 0);
+  rv=_getCertificate(xdlg->provider, u, pid);
   if (rv<0) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "here (%d)", rv);
     return rv;
   }
 
-  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
+  rv=_getBankInfoAnon(xdlg->provider, u, pid);
   if (rv<0) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    DBG_NOTICE(AQHBCI_LOGDOMAIN, "here (%d)", rv);
     return rv;
   }
 
-
-  /* get bank info (for SCA) */
-  DBG_NOTICE(0, "Getting generic bank info");
-  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving generic bank info (SCA)"));
-  ctx=AB_ImExporterContext_new();
-  rv=AH_Provider_GetBankInfo(xdlg->provider, u, ctx, 0 /* without HKTAN */, 0, 1, 0);
+  rv=_getSystemId(xdlg->provider, u, pid);
   if (rv<0) {
-    AB_ImExporterContext_free(ctx);
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    DBG_NOTICE(AQHBCI_LOGDOMAIN, "here (%d)", rv);
     return rv;
   }
-  AB_ImExporterContext_free(ctx);
-
-  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
-  if (rv==GWEN_ERROR_USER_ABORTED) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-    return rv;
-  }
-
-
-  /* get system id */
-  DBG_NOTICE(0, "Getting sysid");
-  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving system id"));
-  ctx=AB_ImExporterContext_new();
-  rv=AH_Provider_GetSysId(xdlg->provider, u, ctx, 0, 1, 0);
-  if (rv<0) {
-    AB_ImExporterContext_free(ctx);
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-    return rv;
-  }
-  AB_ImExporterContext_free(ctx);
-
-  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
-  if (rv==GWEN_ERROR_USER_ABORTED) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-    return rv;
-  }
-
 
   /* select TAN method */
   rv=_selectTanMethod(dlg, u, 0);
@@ -974,12 +742,82 @@ int _reallyDoIt(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid)
     return rv;
   }
 
+  rv=_getAccountList(xdlg->provider, u, pid);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  return 0;
+}
+
+
+
+int _getCertificate(AB_PROVIDER *pro, AB_USER *u, uint32_t pid)
+{
+  int rv;
+
+  DBG_NOTICE(0, "Getting cert (%08x)", AH_User_GetFlags(u));
+  GWEN_Gui_ProgressLog(pid,
+                       GWEN_LoggerLevel_Notice,
+                       I18N("Retrieving SSL certificate"));
+  rv=AH_Provider_GetCert(pro, u, 0, 1, 0);
+  if (rv<0) {
+    DBG_NOTICE(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
+  if (rv==GWEN_ERROR_USER_ABORTED) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  return 0;
+}
+
+
+
+int _getBankInfoAnon(AB_PROVIDER *pro, AB_USER *u, uint32_t pid)
+{
+  AB_IMEXPORTER_CONTEXT *ctx;
+  int rv;
+
+  /* get bank info (for SCA) */
+  DBG_NOTICE(0, "Getting generic bank info");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, "");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving generic bank info (SCA)"));
 
-  /* get account list */
-  DBG_NOTICE(0, "Getting account list");
-  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving account list"));
   ctx=AB_ImExporterContext_new();
-  rv=AH_Provider_GetAccounts(xdlg->provider, u, ctx, 0, 1, 0);
+  rv=AH_Provider_GetBankInfo(pro, u, ctx, 0 /* without HKTAN */, 0, 1, 0);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Error getting bank info (%d), ignoring", rv);
+    GWEN_Gui_ProgressLog(pid,
+                         GWEN_LoggerLevel_Notice,
+                         I18N("This step failed but that's okay, some banks just don't support it."));
+  }
+  AB_ImExporterContext_free(ctx);
+
+  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
+  if (rv==GWEN_ERROR_USER_ABORTED) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  return 0;
+}
+
+
+
+int _getSystemId(AB_PROVIDER *pro, AB_USER *u, uint32_t pid)
+{
+  AB_IMEXPORTER_CONTEXT *ctx;
+  int rv;
+
+  DBG_NOTICE(0, "Getting sysid");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, "");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving system id"));
+  ctx=AB_ImExporterContext_new();
+  rv=AH_Provider_GetSysId(pro, u, ctx, 0, 1, 0);
   if (rv<0) {
     AB_ImExporterContext_free(ctx);
     DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
@@ -993,13 +831,43 @@ int _reallyDoIt(GWEN_DIALOG *dlg, AB_USER *u, uint32_t pid)
     return rv;
   }
 
+  return 0;
+}
+
+
+
+int _getAccountList(AB_PROVIDER *pro, AB_USER *u, uint32_t pid)
+{
+  AB_IMEXPORTER_CONTEXT *ctx;
+  int rv;
+
+  /* get account list */
+  DBG_NOTICE(0, "Getting account list");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, "");
+  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice, I18N("Retrieving account list"));
+  ctx=AB_ImExporterContext_new();
+  rv=AH_Provider_GetAccounts(pro, u, ctx, 0, 1, 0);
+  if (rv<0) {
+    AB_ImExporterContext_free(ctx);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+  AB_ImExporterContext_free(ctx);
+
+  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
+  if (rv==GWEN_ERROR_USER_ABORTED) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
 
   return 0;
 }
 
 
 
-int AH_PinTanDialog_Next(GWEN_DIALOG *dlg)
+
+
+int _dialogNext(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   int page;
@@ -1010,11 +878,11 @@ int AH_PinTanDialog_Next(GWEN_DIALOG *dlg)
 
   page=GWEN_Dialog_GetIntProperty(dlg, "wiz_stack", GWEN_DialogProperty_Value, 0, -1);
   if (page==PAGE_CREATE) {
-    return AH_PinTanDialog_DoIt(dlg);
+    return _addUserAndSetupWithBankServer(dlg);
   }
   else if (page<PAGE_END) {
     page++;
-    return AH_PinTanDialog_EnterPage(dlg, page, 1);
+    return _dialogEnterPage(dlg, page, 1);
   }
   else if (page==PAGE_END)
     return GWEN_DialogEvent_ResultAccept;
@@ -1024,7 +892,7 @@ int AH_PinTanDialog_Next(GWEN_DIALOG *dlg)
 
 
 
-int AH_PinTanDialog_Previous(GWEN_DIALOG *dlg)
+int _dialogPrevious(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   int page;
@@ -1036,7 +904,7 @@ int AH_PinTanDialog_Previous(GWEN_DIALOG *dlg)
   page=GWEN_Dialog_GetIntProperty(dlg, "wiz_stack", GWEN_DialogProperty_Value, 0, -1);
   if (page>PAGE_BEGIN) {
     page--;
-    return AH_PinTanDialog_EnterPage(dlg, page, 0);
+    return _dialogEnterPage(dlg, page, 0);
   }
 
   return GWEN_DialogEvent_ResultHandled;
@@ -1044,7 +912,7 @@ int AH_PinTanDialog_Previous(GWEN_DIALOG *dlg)
 
 
 
-int AH_PinTanDialog_HandleActivatedBankCode(GWEN_DIALOG *dlg)
+int _handleSignalActivatedBankCode(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   GWEN_DIALOG *dlg2;
@@ -1137,7 +1005,7 @@ int AH_PinTanDialog_HandleActivatedBankCode(GWEN_DIALOG *dlg)
 
   GWEN_Dialog_free(dlg2);
 
-  if (AH_PinTanDialog_GetBankPageData(dlg)<0)
+  if (_getBankPageData(dlg)<0)
     GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 0, 0);
   else
     GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 1, 0);
@@ -1147,7 +1015,7 @@ int AH_PinTanDialog_HandleActivatedBankCode(GWEN_DIALOG *dlg)
 
 
 
-int AH_PinTanDialog_HandleActivatedSpecial(GWEN_DIALOG *dlg)
+int _handleSignalActivatedSpecial(GWEN_DIALOG *dlg)
 {
   AH_PINTAN_DIALOG *xdlg;
   GWEN_DIALOG *dlg2;
@@ -1179,7 +1047,7 @@ int AH_PinTanDialog_HandleActivatedSpecial(GWEN_DIALOG *dlg)
     xdlg->httpVMinor=AH_PinTanSpecialDialog_GetHttpVMinor(dlg2);
     xdlg->hbciVersion=AH_PinTanSpecialDialog_GetHbciVersion(dlg2);
     xdlg->flags=AH_PinTanSpecialDialog_GetFlags(dlg2);
-    AH_PinTanDialog_SetTanMediumId(dlg, AH_PinTanSpecialDialog_GetTanMediumId(dlg2));
+    _setTanMediumId(dlg, AH_PinTanSpecialDialog_GetTanMediumId(dlg2));
   }
 
   GWEN_Dialog_free(dlg2);
@@ -1189,19 +1057,19 @@ int AH_PinTanDialog_HandleActivatedSpecial(GWEN_DIALOG *dlg)
 
 
 
-int AH_PinTanDialog_HandleActivated(GWEN_DIALOG *dlg, const char *sender)
+int _handleSignalActivated(GWEN_DIALOG *dlg, const char *sender)
 {
   DBG_NOTICE(0, "Activated: %s", sender);
   if (strcasecmp(sender, "wiz_bankcode_button")==0)
-    return AH_PinTanDialog_HandleActivatedBankCode(dlg);
+    return _handleSignalActivatedBankCode(dlg);
   else if (strcasecmp(sender, "wiz_prev_button")==0)
-    return AH_PinTanDialog_Previous(dlg);
+    return _dialogPrevious(dlg);
   else if (strcasecmp(sender, "wiz_next_button")==0)
-    return AH_PinTanDialog_Next(dlg);
+    return _dialogNext(dlg);
   else if (strcasecmp(sender, "wiz_abort_button")==0)
     return GWEN_DialogEvent_ResultReject;
   else if (strcasecmp(sender, "wiz_special_button")==0)
-    return AH_PinTanDialog_HandleActivatedSpecial(dlg);
+    return _handleSignalActivatedSpecial(dlg);
   else if (strcasecmp(sender, "wiz_help_button")==0) {
     /* TODO: open a help dialog */
   }
@@ -1211,7 +1079,7 @@ int AH_PinTanDialog_HandleActivated(GWEN_DIALOG *dlg, const char *sender)
 
 
 
-int AH_PinTanDialog_HandleValueChanged(GWEN_DIALOG *dlg, const char *sender)
+int _handleSignalValueChanged(GWEN_DIALOG *dlg, const char *sender)
 {
   if (strcasecmp(sender, "wiz_bankcode_edit")==0 ||
       strcasecmp(sender, "wiz_url_edit")==0 ||
@@ -1221,14 +1089,14 @@ int AH_PinTanDialog_HandleValueChanged(GWEN_DIALOG *dlg, const char *sender)
     int rv;
 
     if (GWEN_Dialog_GetIntProperty(dlg, "wiz_stack", GWEN_DialogProperty_Value, 0, -1)==PAGE_BANK) {
-      rv=AH_PinTanDialog_GetBankPageData(dlg);
+      rv=_getBankPageData(dlg);
       if (rv<0)
         GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 0, 0);
       else
         GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 1, 0);
     }
     else if (GWEN_Dialog_GetIntProperty(dlg, "wiz_stack", GWEN_DialogProperty_Value, 0, -1)==PAGE_USER) {
-      rv=AH_PinTanDialog_GetUserPageData(dlg);
+      rv=_getUserPageData(dlg);
       if (rv<0)
         GWEN_Dialog_SetIntProperty(dlg, "wiz_next_button", GWEN_DialogProperty_Enabled, 0, 0, 0);
       else
@@ -1241,9 +1109,7 @@ int AH_PinTanDialog_HandleValueChanged(GWEN_DIALOG *dlg, const char *sender)
 
 
 
-int GWENHYWFAR_CB AH_PinTanDialog_SignalHandler(GWEN_DIALOG *dlg,
-                                                GWEN_DIALOG_EVENTTYPE t,
-                                                const char *sender)
+int GWENHYWFAR_CB _dlgApi_SignalHandler(GWEN_DIALOG *dlg, GWEN_DIALOG_EVENTTYPE t, const char *sender)
 {
   AH_PINTAN_DIALOG *xdlg;
 
@@ -1253,24 +1119,25 @@ int GWENHYWFAR_CB AH_PinTanDialog_SignalHandler(GWEN_DIALOG *dlg,
 
   switch (t) {
   case GWEN_DialogEvent_TypeInit:
-    AH_PinTanDialog_Init(dlg);
+    _dialogInit(dlg);
     return GWEN_DialogEvent_ResultHandled;;
 
   case GWEN_DialogEvent_TypeFini:
-    AH_PinTanDialog_Fini(dlg);
+    _dialogFini(dlg);
     return GWEN_DialogEvent_ResultHandled;;
 
   case GWEN_DialogEvent_TypeValueChanged:
-    return AH_PinTanDialog_HandleValueChanged(dlg, sender);
+    return _handleSignalValueChanged(dlg, sender);
 
   case GWEN_DialogEvent_TypeActivated:
-    return AH_PinTanDialog_HandleActivated(dlg, sender);
+    return _handleSignalActivated(dlg, sender);
 
   case GWEN_DialogEvent_TypeEnabled:
   case GWEN_DialogEvent_TypeDisabled:
   case GWEN_DialogEvent_TypeClose:
 
   case GWEN_DialogEvent_TypeLast:
+  default:
     return GWEN_DialogEvent_ResultNotHandled;
 
   }
@@ -1310,4 +1177,19 @@ int _selectTanMethod(GWEN_DIALOG *dlg, AB_USER *u, int doLock)
 
 
 
+void _removeAllSpaces(uint8_t *s)
+{
+  uint8_t *d;
+
+  d=s;
+  while (*s) {
+    if (*s>33)
+      *(d++)=*s;
+    s++;
+  }
+  *d=0;
+}
+
+
+
 
